// IR Decoder
//
// Receives simple NEC IR commands and displays them on an LCD
// IR data is received as a pin change interrupt from a standard 38-kHz IR detector.
//     Start: 9ms low pulse
//     Space: 4.5ms high pulse
//     Address: 13.5ms (8 data bits)
//     Inverted Address: 13.5ms (8 data bits)
//     Command: 13.5ms (8 data bits)
//     Inverted Command: 13.5ms (8 data bits)
//     Data bit = 0: 562us low pulse followed by 562.5us high pulse
//     Data bit = 1: 562us low pulse followed by 1.6875ms high pulse
//
// Written by Boomer48: February 2021

#include "LCD_4Bit.h"

#define IR_Input 11
#define Sync_Time 3000
#define One_Time 1000
#define Zero_Time 400
volatile byte Bit_Count = 0;
volatile byte Byte_Count = 0;
volatile byte Byte_Ready = false;
volatile byte IR_Bytes[4];
unsigned long Start_Time = 0;
unsigned long Pulse_Width;
char ASCII[] = "0123456789ABCDEF";

//----------------------------------------------------------------------
void setup()  {
  LCD_setup();

  delay(100); // wait for LCD to get ready
  Send_Byte('A');
  Send_Byte('D');
  Send_Byte('D');
  Send_Byte('R');
  Send_Byte('E');
  Send_Byte('S');
  Send_Byte('S');
  Send_Byte('/');
  Send_Byte('C');
  Send_Byte('O');
  Send_Byte('M');
  Send_Byte('M');
  Send_Byte('A');
  Send_Byte('N');
  Send_Byte('D');
  LCD_Line2();

  // Inputs between 8 and 13, inclusive, use PCINT0
  bitSet(PCMSK0, IR_Input - 8);
  PCICR = B00000001; // enable PCIE0 group
}

void loop() {
  if (Byte_Ready == true) {
    Byte_Ready = false;
    Byte_Count++;

    if (Byte_Count == 4) {
      // got all four data bytes
      Byte_Count = 0;
      LCD_Line2(); // reset LCD write pointer to start of line

      for (byte i = 0; i < 4; i++)
      {
        Send_Byte(ASCII[IR_Bytes[i] >> 4]); // display upper nibble first
        Send_Byte(ASCII[IR_Bytes[i] & 0x0F]);
        Send_Byte(' ');
        Send_Byte(' ');
      }
    }
  }
} // end loop

// Interrupt-on-change handler
ISR (PCINT0_vect)
{
  //when the pin goes HIGH record the pulse start time
  if (digitalRead(IR_Input) == HIGH) {
    Start_Time = micros();
  }
  else // pin went low
    if (Start_Time != 0) {
      //calculate the pulse width
      Pulse_Width = micros() - Start_Time;
      //clear the timer
      Start_Time = 0;

      if (Pulse_Width > Sync_Time) {
        // sync bit
        Byte_Ready = false;
        Bit_Count = 0;
        Byte_Count = 0;
      }
      else if (Pulse_Width > One_Time) {
        // data bit = 1
        bitSet(IR_Bytes[Byte_Count], Bit_Count);
        Bit_Count++;
      }
      else if (Pulse_Width > Zero_Time) {
        // data bit = 0
        bitClear(IR_Bytes[Byte_Count], Bit_Count);
        Bit_Count++;
      }
      else
        Bit_Count = 0; // error

      if (Bit_Count == 8) {
        Byte_Ready = true;
        Bit_Count = 0;
      }
    }
}
